import 'package:petitparser/petitparser.dart';

/******************************************************************************
 * These are the unit types that can be used in FHIRPath using the UCUM
 * specification (which has not yet been fully implemented)
 * ***************************************************************************/

final durationLexer = (string('milliseconds') |
    string('seconds') |
    string('minutes') |
    string('hours') |
    string('days') |
    string('weeks') |
    string('months') |
    string('years') |
    string('millisecond') |
    string('second') |
    string('minute') |
    string('hour') |
    string('day') |
    string('week') |
    string('month') |
    string('year'));

// any code in curly braces is a valid UCUM comment
final unitLexer = (string("'{") & (char('}').neg()).plus() & string("}'")) |
    (string("'1'") |
        string("'m'") |
        string("'s'") |
        string("'g'") |
        string("'rad'") |
        string("'K'") |
        string("'C'") |
        string("'cd'") |
        string("'10*'") |
        string("'10^'") |
        string("'[pi]'") |
        string("'%'") |
        string("'[ppth]'") |
        string("'[ppm]'") |
        string("'[ppb]'") |
        string("'[pptr]'") |
        string("'mol'") |
        string("'sr'") |
        string("'Hz'") |
        string("'N'") |
        string("'Pa'") |
        string("'J'") |
        string("'W'") |
        string("'A'") |
        string("'V'") |
        string("'F'") |
        string("'Ohm'") |
        string("'S'") |
        string("'Wb'") |
        string("'Cel'") |
        string("'T'") |
        string("'H'") |
        string("'lm'") |
        string("'lx'") |
        string("'Bq'") |
        string("'Gy'") |
        string("'Sv'") |
        string("'deg'") |
        string("'gon'") |
        string("'l'") |
        string("'L'") |
        string("'ar'") |
        string("'min'") |
        string("'h'") |
        string("'d'") |
        string("'a_t'") |
        string("'a_j'") |
        string("'a_g'") |
        string("'a'") |
        string("'wk'") |
        string("'mo_s'") |
        string("'mo_j'") |
        string("'mo_g'") |
        string("'mo'") |
        string("'t'") |
        string("'bar'") |
        string("'u'") |
        string("'AU'") |
        string("'pc'") |
        string("'[c]'") |
        string("'[h]'") |
        string("'[k]'") |
        string("'[eps_0]'") |
        string("'[mu_0]'") |
        string("'[e]'") |
        string("'eV'") |
        string("'[m_e]'") |
        string("'[m_p]'") |
        string("'[G]'") |
        string("'[g]'") |
        string("'Torr'") |
        string("'atm'") |
        string("'[ly]'") |
        string("'gf'") |
        string("'Ky'") |
        string("'Gal'") |
        string("'dyn'") |
        string("'erg'") |
        string("'P'") |
        string("'Bi'") |
        string("'St'") |
        string("'Mx'") |
        string("'G'") |
        string("'Oe'") |
        string("'Gb'") |
        string("'sb'") |
        string("'Lmb'") |
        string("'ph'") |
        string("'Ci'") |
        string("'R'") |
        string("'RAD'") |
        string("'REM'") |
        string("'[in_i]'") |
        string("'[ft_i]'") |
        string("'[yd_i]'") |
        string("'[mi_i]'") |
        string("'[fth_i]'") |
        string("'[nmi_i]'") |
        string("'[kn_i]'") |
        string("'[sin_i]'") |
        string("'[sft_i]'") |
        string("'[syd_i]'") |
        string("'[cin_i]'") |
        string("'[cft_i]'") |
        string("'[cyd_i]'") |
        string("'[bf_i]'") |
        string("'[cr_i]'") |
        string("'[mil_i]'") |
        string("'[cml_i]'") |
        string("'[hd_i]'") |
        string("'[ft_us]'") |
        string("'[yd_us]'") |
        string("'[in_us]'") |
        string("'[rd_us]'") |
        string("'[ch_us]'") |
        string("'[lk_us]'") |
        string("'[rch_us]'") |
        string("'[rlk_us]'") |
        string("'[fth_us]'") |
        string("'[fur_us]'") |
        string("'[mi_us]'") |
        string("'[acr_us]'") |
        string("'[srd_us]'") |
        string("'[smi_us]'") |
        string("'[sct]'") |
        string("'[twp]'") |
        string("'[mil_us]'") |
        string("'[in_br]'") |
        string("'[ft_br]'") |
        string("'[rd_br]'") |
        string("'[ch_br]'") |
        string("'[lk_br]'") |
        string("'[fth_br]'") |
        string("'[pc_br]'") |
        string("'[yd_br]'") |
        string("'[mi_br]'") |
        string("'[nmi_br]'") |
        string("'[kn_br]'") |
        string("'[acr_br]'") |
        string("'[gal_us]'") |
        string("'[bbl_us]'") |
        string("'[qt_us]'") |
        string("'[pt_us]'") |
        string("'[gil_us]'") |
        string("'[foz_us]'") |
        string("'[fdr_us]'") |
        string("'[min_us]'") |
        string("'[crd_us]'") |
        string("'[bu_us]'") |
        string("'[gal_wi]'") |
        string("'[pk_us]'") |
        string("'[dqt_us]'") |
        string("'[dpt_us]'") |
        string("'[tbs_us]'") |
        string("'[tsp_us]'") |
        string("'[cup_us]'") |
        string("'[foz_m]'") |
        string("'[cup_m]'") |
        string("'[tsp_m]'") |
        string("'[tbs_m]'") |
        string("'[gal_br]'") |
        string("'[pk_br]'") |
        string("'[bu_br]'") |
        string("'[qt_br]'") |
        string("'[pt_br]'") |
        string("'[gil_br]'") |
        string("'[foz_br]'") |
        string("'[fdr_br]'") |
        string("'[min_br]'") |
        string("'[gr]'") |
        string("'[lb_av]'") |
        string("'[lbf_av]'") |
        string("'[oz_av]'") |
        string("'[dr_av]'") |
        string("'[scwt_av]'") |
        string("'[lcwt_av]'") |
        string("'[ston_av]'") |
        string("'[lton_av]'") |
        string("'[stone_av]'") |
        string("'[pwt_tr]'") |
        string("'[oz_tr]'") |
        string("'[lb_tr]'") |
        string("'[sc_ap]'") |
        string("'[dr_ap]'") |
        string("'[oz_ap]'") |
        string("'[lb_ap]'") |
        string("'[oz_m]'") |
        string("'[lne]'") |
        string("'[pnt]'") |
        string("'[pca]'") |
        string("'[pnt_pr]'") |
        string("'[pca_pr]'") |
        string("'[pied]'") |
        string("'[pouce]'") |
        string("'[ligne]'") |
        string("'[didot]'") |
        string("'[cicero]'") |
        string("'[degF]'") |
        string("'[degR]'") |
        string("'[degRe]'") |
        string("'cal_[15]'") |
        string("'cal_[20]'") |
        string("'cal_m'") |
        string("'cal_IT'") |
        string("'cal_th'") |
        string("'cal'") |
        string("'[Cal]'") |
        string("'[Btu_39]'") |
        string("'[Btu_59]'") |
        string("'[Btu_60]'") |
        string("'[Btu_m]'") |
        string("'[Btu_IT]'") |
        string("'[Btu_th]'") |
        string("'[Btu]'") |
        string("'[HP]'") |
        string("'tex'") |
        string("'[den]'") |
        string("'m[H2O]'") |
        string("'m[Hg]'") |
        string("'[in_i'H2O]'") |
        string("'[in_i'Hg]'") |
        string("'[PRU]'") |
        string("'[wood'U]'") |
        string("[diop]'") |
        string("'[p'diop]'") |
        string("'%[slope]'") |
        string("'[mesh_i]'") |
        string("'[Ch]'") |
        string("'[drp]'") |
        string("'[hnsf'U]'") |
        string("'[MET]'") |
        string("[hp'_X]'") |
        string("'[hp'_C]'") |
        string("'[hp'_M]'") |
        string("'[hp'_Q]'") |
        string("'[hp_X]'") |
        string("'[hp_C]'") |
        string("'[hp_M]'") |
        string("'[hp_Q]'") |
        string("'[kp_X]'") |
        string("'[kp_C]'") |
        string("'[kp_M]'") |
        string("'[kp_Q]'") |
        string("'eq'") |
        string("'osm'") |
        string("'[pH]'") |
        string("'g%'") |
        string("'[S]'") |
        string("'[HPF]'") |
        string("'[LPF]'") |
        string("'kat'") |
        string("'U'") |
        string("'[iU]'") |
        string("'[IU]'") |
        string("'[arb'U]'") |
        string("'[USP'U]'") |
        string("'[GPL'U]'") |
        string("'[MPL'U]'") |
        string("'[APL'U]'") |
        string("'[beth'U]'") |
        string("'[anti'Xa'U]'") |
        string("'[todd'U]'") |
        string("'[dye'U]'") |
        string("'[smgy'U]'") |
        string("'[bdsk'U]'") |
        string("'[ka'U]'") |
        string("'[knk'U]'") |
        string("'[mclg'U]'") |
        string("'[tb'U]'") |
        string("'[CCID_50]'") |
        string("'[TCID_50]'") |
        string("'[EID_50]'") |
        string("'[PFU]'") |
        string("'[FFU]'") |
        string("'[CFU]'") |
        string("'[IR]'") |
        string("'[BAU]'") |
        string("'[AU]'") |
        string("'Amb'a'1'U]'") |
        string("'[PNU]'") |
        string("'[Lf]'") |
        string("'[D'ag'U]'") |
        string("'[FEU]'") |
        string("'[ELU]'") |
        string("'[EU]'") |
        string("'Np'") |
        string("'B'") |
        string("'B[SPL]'") |
        string("'B[V]'") |
        string("'B[mV]'") |
        string("'B[uV]'") |
        string("'B[10.nV]'") |
        string("'B[W]'") |
        string("'B[kW]'") |
        string("'st'") |
        string("'Ao'") |
        string("'b'") |
        string("'att'") |
        string("'mho'") |
        string("'[psi]'") |
        string("'circ'") |
        string("'sph'") |
        string("'[car_m]'") |
        string("'[car_Au]'") |
        string("'[smoot]'") |
        string("'[m/s2/Hz^(1/2)]'") |
        string("'bit_s'") |
        string("'bit'") |
        string("'By'") |
        string("'Bd'") |
        string("'/(12.h)'") |
        string("'/[arb'U]'") |
        string("'/[HPF]'") |
        string("'/[IU]'") |
        string("'/[LPF]'") |
        string("'/10*10'") |
        string("'/10*12'") |
        string("'/10*3'") |
        string("'/10*6'") |
        string("'/10*9'") |
        string("'/100'") |
        string("'/100{cells}'") |
        string("'/100{neutrophils}'") |
        string("'/100{spermatozoa}'") |
        string("'/100{WBCs}'") |
        string("'/a'") |
        string("'/cm[H2O]'") |
        string("'/d'") |
        string("'/dL'") |
        string("'/g'") |
        string("'/h'") |
        string("'/kg'") |
        string("'/L'") |
        string("'/m2'") |
        string("'/m3'") |
        string("'/mg'") |
        string("'/min'") |
        string("'/mL'") |
        string("'/mm'") |
        string("'/mo'") |
        string("'/s'") |
        string("'/U'") |
        string("'/uL'") |
        string("'/wk'") |
        string("'[APL'U]/mL'") |
        string("'[arb'U]/mL'") |
        string("'[CFU]/L'") |
        string("'[CFU]/mL'") |
        string("'[ft_us]/[ft_us]'") |
        string("'[GPL'U]/mL'") |
        string("'[IU]/(2.h)'") |
        string("'[IU]/(24.h)'") |
        string("'[IU]/d'") |
        string("'[IU]/dL'") |
        string("'[IU]/g'") |
        string("'[IU]/h'") |
        string("'[IU]/kg'") |
        string("'[IU]/kg/d'") |
        string("'[IU]/L'") |
        string("'[IU]/min'") |
        string("'[IU]/mL'") |
        string("'[MPL'U]/mL'") |
        string("'{#}/[HPF]'") |
        string("'{#}/[LPF]'") |
        string("'{APS'U}'") |
        string("'{EIA_index}'") |
        string("'{KCT'U}'") |
        string("'{MPS'U}'") |
        string("'10*12/L'") |
        string("'10*3'") |
        string("'10*3/L'") |
        string("'10*3/mL'") |
        string("'10*3/uL'") |
        string("'10*4/uL'") |
        string("'10*5'") |
        string("'10*6'") |
        string("'10*6.[CFU]/L'") |
        string("'10*6.[IU]'") |
        string("'10*6/(24.h)'") |
        string("'10*6/kg'") |
        string("'10*6/L'") |
        string("'10*6/mL'") |
        string("'10*6/uL'") |
        string("'10*8'") |
        string("'10*9/L'") |
        string("'10*9/mL'") |
        string("'10*9/uL'") |
        string("'10.L/(min.m2)'") |
        string("'10.L/min'") |
        string("'10.uN.s/(cm5.m2)'") |
        string("'24.h'") |
        string("'A/m'") |
        string("'cg'") |
        string("'cL'") |
        string("'cm'") |
        string("'cm[H2O]'") |
        string("'cm[H2O]/L/s'") |
        string("'cm[H2O]/s/m'") |
        string("'cm[Hg]'") |
        string("'cm2'") |
        string("'cm2/s'") |
        string("'cP'") |
        string("'cSt'") |
        string("'daL/min'") |
        string("'daL/min/m2'") |
        string("'dB'") |
        string("'deg/s'") |
        string("'dg'") |
        string("'dL'") |
        string("'dm'") |
        string("'dm2/s2'") |
        string("'dyn.s/(cm.m2)'") |
        string("'dyn.s/cm'") |
        string("'eq/L'") |
        string("'eq/mL'") |
        string("'eq/mmol'") |
        string("'eq/umol'") |
        string("'fg'") |
        string("'fL'") |
        string("'fm'") |
        string("'fmol'") |
        string("'fmol/g'") |
        string("'fmol/L'") |
        string("'fmol/mg'") |
        string("'fmol/mL'") |
        string("'g.m'") |
        string("'g/(100.g)'") |
        string("'g/(12.h)'") |
        string("'g/(24.h)'") |
        string("'g/(3.d)'") |
        string("'g/(4.h)'") |
        string("'g/(48.h)'") |
        string("'g/(5.h)'") |
        string("'g/(6.h)'") |
        string("'g/(72.h)'") |
        string("'g/cm3'") |
        string("'g/d'") |
        string("'g/dL'") |
        string("'g/g'") |
        string("'g/h'") |
        string("'g/h/m2'") |
        string("'g/kg '") |
        string("'g/kg/(8.h)'") |
        string("'g/kg/d'") |
        string("'g/kg/h'") |
        string("'g/kg/min'") |
        string("'g/L'") |
        string("'g/m2'") |
        string("'g/mg'") |
        string("'g/min'") |
        string("'g/mL'") |
        string("'g/mmol'") |
        string("'J/L'") |
        string("'K/W'") |
        string("'k[IU]/L'") |
        string("'k[IU]/mL'") |
        string("'kat/kg'") |
        string("'kat/L'") |
        string("'kcal'") |
        string("'kcal/(24.h)'") |
        string("'kcal/[oz_av]'") |
        string("'kcal/d'") |
        string("'kcal/h'") |
        string("'kcal/kg/(24.h)'") |
        string("'kg'") |
        string("'kg.m/s'") |
        string("'kg/(s.m2)'") |
        string("'kg/h'") |
        string("'kg/L'") |
        string("'kg/m2'") |
        string("'kg/m3'") |
        string("'kg/min'") |
        string("'kg/mol'") |
        string("'kg/s'") |
        string("'kL'") |
        string("'km'") |
        string("'kPa'") |
        string("'ks'") |
        string("'kU'") |
        string("'kU/g'") |
        string("'kU/L'") |
        string("'kU/mL'") |
        string("'L/(24.h)'") |
        string("'L/(8.h)'") |
        string("'L/(min.m2) '") |
        string("'L/d'") |
        string("'L/h'") |
        string("'L/kg'") |
        string("'L/L'") |
        string("'L/min'") |
        string("'L/min/m2'") |
        string("'L/s'") |
        string("'L/s/s2'") |
        string("'lm.m2'") |
        string("'m/s'") |
        string("'m/s2'") |
        string("'m[IU]/L'") |
        string("'m[IU]/mL'") |
        string("'m2'") |
        string("'m2/s'") |
        string("'m3/s'") |
        string("'mA'") |
        string("'mbar'") |
        string("'mbar.s/L'") |
        string("'mbar/L/s'") |
        string("'meq'") |
        string("'meq/(2.h)'") |
        string("'meq/(24.h)'") |
        string("'meq/(8.h)'") |
        string("'meq/d'") |
        string("'meq/dL'") |
        string("'meq/g'") |
        string("'meq/h'") |
        string("'meq/kg'") |
        string("'meq/kg/h'") |
        string("'meq/L'") |
        string("'meq/m2'") |
        string("'meq/min'") |
        string("'meq/mL'") |
        string("'mg'") |
        string("'mg/(10.h)'") |
        string("'mg/(12.h)'") |
        string("'mg/(2.h)'") |
        string("'mg/(24.h)'") |
        string("'mg/(6.h)'") |
        string("'mg/(72.h)'") |
        string("'mg/(8.h)'") |
        string("'mg/d'") |
        string("'mg/dL'") |
        string("'mg/g'") |
        string("'mg/h'") |
        string("'mg/kg'") |
        string("'mg/kg/(8.h)'") |
        string("'mg/kg/d'") |
        string("'mg/kg/h'") |
        string("'mg/kg/min'") |
        string("'mg/L'") |
        string("'mg/m2'") |
        string("'mg/m3'") |
        string("'mg/mg'") |
        string("'mg/min'") |
        string("'mg/mL'") |
        string("'mg/mmol'") |
        string("'mg/wk'") |
        string("'mL'") |
        string("'mL/(10.h)'") |
        string("'mL/(12.h)'") |
        string("'mL/(2.h)'") |
        string("'mL/(24.h)'") |
        string("'mL/(4.h)'") |
        string("'mL/(5.h)'") |
        string("'mL/(6.h)'") |
        string("'mL/(72.h)'") |
        string("'mL/(8.h)'") |
        string("'mL/(8.h)/kg'") |
        string("'mL/[sin_i]'") |
        string("'mL/cm[H2O]'") |
        string("'mL/d'") |
        string("'mL/dL'") |
        string("'mL/h'") |
        string("'mL/kg'") |
        string("'mL/kg/(8.h)'") |
        string("'mL/kg/d'") |
        string("'mL/kg/h'") |
        string("'mL/kg/min'") |
        string("'mL/m2'") |
        string("'mL/mbar'") |
        string("'mL/min'") |
        string("'mL/min/m2'") |
        string("'mL/mm'") |
        string("'mL/s'") |
        string("'mm'") |
        string("'mm/h'") |
        string("'mm/min'") |
        string("'mm[H2O]'") |
        string("'mm[Hg]'") |
        string("'mm2'") |
        string("'mmol'") |
        string("'mmol/(12.h)'") |
        string("'mmol/(2.h)'") |
        string("'mmol/(24.h)'") |
        string("'mmol/(5.h)'") |
        string("'mmol/(6.h)'") |
        string("'mmol/(8.h)'") |
        string("'mmol/d'") |
        string("'mmol/dL'") |
        string("'mmol/g'") |
        string("'mmol/h'") |
        string("'mmol/kg'") |
        string("'mmol/kg/(8.h)'") |
        string("'mmol/kg/d'") |
        string("'mmol/kg/h'") |
        string("'mmol/kg/min'") |
        string("'mmol/L'") |
        string("'mmol/m2'") |
        string("'mmol/min'") |
        string("'mmol/mmol'") |
        string("'mmol/mol'") |
        string("'mmol/s/L'") |
        string("'mol/kg'") |
        string("'mol/kg/s'") |
        string("'mol/L'") |
        string("'mol/m3'") |
        string("'mol/mL'") |
        string("'mol/mol'") |
        string("'mol/s'") |
        string("'mosm'") |
        string("'mosm/kg'") |
        string("'mosm/L'") |
        string("'mPa'") |
        string("'mPa.s'") |
        string("'Ms'") |
        string("'ms'") |
        string("'mU/g'") |
        string("'mU/L'") |
        string("'mU/mg'") |
        string("'mU/mL'") |
        string("'mU/mL/min'") |
        string("'mV'") |
        string("'N.cm'") |
        string("'N.s'") |
        string("'ng'") |
        string("'ng/(24.h)'") |
        string("'ng/(8.h)'") |
        string("'ng/10*6'") |
        string("'ng/d'") |
        string("'ng/dL'") |
        string("'ng/g'") |
        string("'ng/h'") |
        string("'ng/kg'") |
        string("'ng/kg/(8.h)'") |
        string("'ng/kg/h'") |
        string("'ng/kg/min'") |
        string("'ng/L'") |
        string("'ng/m2'") |
        string("'ng/mg'") |
        string("'ng/mg/h'") |
        string("'ng/min'") |
        string("'ng/mL'") |
        string("'ng/mL/h'") |
        string("'ng/s'") |
        string("'ng/U'") |
        string("'nkat'") |
        string("'nL'") |
        string("'nm'") |
        string("'nm/s/L'") |
        string("'nmol'") |
        string("'nmol/(24.h)'") |
        string("'nmol/d'") |
        string("'nmol/dL'") |
        string("'nmol/g'") |
        string("'nmol/h/L'") |
        string("'nmol/L'") |
        string("'nmol/mg'") |
        string("'nmol/mg/h'") |
        string("'nmol/mg{prot}'") |
        string("'nmol/min'") |
        string("'nmol/min/mL'") |
        string("'nmol/mL'") |
        string("'nmol/mL/h'") |
        string("'nmol/mL/min'") |
        string("'nmol/mmol'") |
        string("'nmol/mmol{creat}'") |
        string("'nmol/mol'") |
        string("'nmol/nmol'") |
        string("'nmol/s'") |
        string("'nmol/s/L'") |
        string("'ns'") |
        string("'nU/mL'") |
        string("'Ohm.m'") |
        string("'osm/kg'") |
        string("'osm/L'") |
        string("'pA'") |
        string("'pg'") |
        string("'pg/dL'") |
        string("'pg/L'") |
        string("'pg/mg'") |
        string("'pg/mL'") |
        string("'pg/mm'") |
        string("'pkat'") |
        string("'pL'") |
        string("'pm'") |
        string("'pmol'") |
        string("'pmol/(24.h)'") |
        string("'pmol/d'") |
        string("'pmol/dL'") |
        string("'pmol/g'") |
        string("'pmol/h/mL'") |
        string("'pmol/L'") |
        string("'pmol/min'") |
        string("'pmol/mL'") |
        string("'pmol/umol'") |
        string("'ps'") |
        string("'pT'") |
        string("'U/(12.h)'") |
        string("'U/(2.h)'") |
        string("'U/(24.h)'") |
        string("'U/10'") |
        string("'U/10*10'") |
        string("'U/10*12'") |
        string("'U/10*6'") |
        string("'U/10*9'") |
        string("'U/d'") |
        string("'U/dL'") |
        string("'U/g'") |
        string("'U/h'") |
        string("'U/L'") |
        string("'U/min'") |
        string("'U/mL'") |
        string("'U/s'") |
        string("'u[IU]'") |
        string("'u[IU]/L'") |
        string("'u[IU]/mL'") |
        string("'ueq'") |
        string("'ueq/L'") |
        string("'ueq/mL'") |
        string("'ug'") |
        string("'ug/(100.g)'") |
        string("'ug/(24.h)'") |
        string("'ug/(8.h)'") |
        string("'ug/[sft_i]'") |
        string("'ug/d'") |
        string("'ug/dL'") |
        string("'ug/g'") |
        string("'ug/h'") |
        string("'ug/kg'") |
        string("'ug/kg/(8.h)'") |
        string("'ug/kg/d'") |
        string("'ug/kg/h'") |
        string("'ug/kg/min'") |
        string("'ug/L'") |
        string("'ug/L/(24.h)'") |
        string("'ug/m2'") |
        string("'ug/m3'") |
        string("'ug/mg'") |
        string("'ug/min'") |
        string("'ug/mL'") |
        string("'ug/mmol'") |
        string("'ug/ng'") |
        string("'ukat'") |
        string("'uL'") |
        string("'uL/(2.h)'") |
        string("'uL/h'") |
        string("'um'") |
        string("'um/s'") |
        string("'umol'") |
        string("'umol/(2.h)'") |
        string("'umol/(24.h)'") |
        string("'umol/(8.h)'") |
        string("'umol/d'") |
        string("'umol/dL'") |
        string("'umol/g'") |
        string("'umol/h'") |
        string("'umol/kg'") |
        string("'umol/L'") |
        string("'umol/L/h'") |
        string("'umol/mg'") |
        string("'umol/min'") |
        string("'umol/min/g'") |
        string("'umol/min/L'") |
        string("'umol/mL'") |
        string("'umol/mL/min'") |
        string("'umol/mmol'") |
        string("'umol/mol'") |
        string("'umol/umol'") |
        string("'uOhm'") |
        string("'us'") |
        string("'uU/g'") |
        string("'uU/L'") |
        string("'uU/mL'") |
        string("'uV'") |
        // minute: since this is an apostrophe, it can only be used in escaped form.
        // TODO: Figure out proper escape sequence
        string(r"\\'\\'") |
        // second: since these are apostrophes, it can only be used in escaped form.
        // TODO: Figure out proper escape sequence
        string(r"\\''\\'") |
        // an empty unit = '1'
        string("''"));
